@@ -9,11 +9,12 @@ for IO related test cases.
 """
 
 import unittest, doctest
-import os, shutil, tempfile, copy
+import os, re, shutil, tempfile, copy
 
-from common_imports import StringIO, etree, ElementTree, HelperTestCase, fileInTestDir, canonicalize
+from common_imports import StringIO, etree, ElementTree
+from common_imports import HelperTestCase, fileInTestDir, canonicalize
 
-class ETreeTestCaseBase(unittest.TestCase):
+class ETreeTestCaseBase(HelperTestCase):
     etree = None
     
     def setUp(self):
@@ -89,6 +90,24 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('two', root[1].tag)
         self.assertEquals('three', root[2].tag)
         
+    def test_element_contains(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        root1 = Element('root')
+        SubElement(root1, 'one')
+        self.assert_(root1[0] in root1)
+
+        root2 = Element('root')
+        SubElement(root2, 'two')
+        SubElement(root2, 'three')
+        self.assert_(root2[0] in root2)
+        self.assert_(root2[1] in root2)
+
+        self.assertFalse(root1[0] in root2)
+        self.assertFalse(root2[0] in root1)
+        self.assertFalse(None in root2)
+
     def test_element_indexing_with_text(self):
         ElementTree = self.etree.ElementTree
         
@@ -169,6 +188,42 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals(None, root.text)
         self.assertEquals('One', root[0].text)
 
+    def test_text_escape_in(self):
+        ElementTree = self.etree.ElementTree
+
+        f = StringIO('<doc>This is &gt; than a text</doc>')
+        doc = ElementTree(file=f)
+        root = doc.getroot()
+        self.assertEquals('This is > than a text', root.text)
+
+    def test_text_escape_out(self):
+        Element = self.etree.Element
+
+        a = Element("a")
+        a.text = "<>&"
+        self.assertXML('<a>&lt;&gt;&amp;</a>',
+                       a)
+
+    def test_text_escape_tostring(self):
+        tostring = self.etree.tostring
+        Element  = self.etree.Element
+
+        a = Element("a")
+        a.text = "<>&"
+        self.assertEquals('<a>&lt;&gt;&amp;</a>',
+                         tostring(a))
+
+    def test_text_str_subclass(self):
+        Element = self.etree.Element
+
+        class strTest(str):
+            pass
+
+        a = Element("a")
+        a.text = strTest("text")
+        self.assertXML('<a>text</a>',
+                       a)
+
     def test_tail(self):
         ElementTree = self.etree.ElementTree
         
@@ -181,6 +236,18 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('mixed', root[0].text)
         self.assertEquals(' content.', root[0].tail)
 
+    def test_tail_str_subclass(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        class strTest(str):
+            pass
+
+        a = Element("a")
+        SubElement(a, "t").tail = strTest("tail")
+        self.assertXML('<a><t></t>tail</a>',
+                       a)
+
     def test_ElementTree(self):
         Element = self.etree.Element
         ElementTree = self.etree.ElementTree
@@ -223,6 +290,60 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals(None, root.get('three'))
         self.assertEquals('foo', root.get('three', 'foo'))
 
+    def test_attribute_update_dict(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta"/>')
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'Alpha'), ('beta', 'Beta')],
+            items)
+
+        root.attrib.update({'alpha' : 'test', 'gamma' : 'Gamma'})
+
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'test'), ('beta', 'Beta'), ('gamma', 'Gamma')],
+            items)
+
+    def test_attribute_update_sequence(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta"/>')
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'Alpha'), ('beta', 'Beta')],
+            items)
+
+        root.attrib.update({'alpha' : 'test', 'gamma' : 'Gamma'}.items())
+
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'test'), ('beta', 'Beta'), ('gamma', 'Gamma')],
+            items)
+
+    def test_attribute_update_iter(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta"/>')
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'Alpha'), ('beta', 'Beta')],
+            items)
+
+        root.attrib.update({'alpha' : 'test', 'gamma' : 'Gamma'}.iteritems())
+
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'test'), ('beta', 'Beta'), ('gamma', 'Gamma')],
+            items)
+
     def test_attribute_keys(self):
         XML = self.etree.XML
         
@@ -325,6 +446,13 @@ class ETreeTestCaseBase(unittest.TestCase):
             True,
             '{http://ns.codespeak.net/test}baz' in root.attrib)
 
+    def test_attribute_set(self):
+        Element = self.etree.Element
+
+        root = Element("root")
+        root.set("attr", "TEST")
+        self.assertEquals("TEST", root.get("attr"))
+
     def test_XML(self):
         XML = self.etree.XML
         
@@ -369,6 +497,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         ElementTree = self.etree.ElementTree
         XML = self.etree.XML
         Comment = self.etree.Comment
+        ProcessingInstruction = self.etree.ProcessingInstruction
         
         el = Element('hoi')
         self.assert_(iselement(el))
@@ -382,6 +511,9 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         c = Comment('test')
         self.assert_(iselement(c))
+
+        p = ProcessingInstruction("test", "some text")
+        self.assert_(iselement(p))
         
     def test_iteration(self):
         XML = self.etree.XML
@@ -545,6 +677,14 @@ class ETreeTestCaseBase(unittest.TestCase):
             self.assertEquals(
                 '<doc%s>This is a test.</doc%s>' % (i, i),
                 canonicalize(data))
+        
+    def test_write_fail(self):
+        ElementTree = self.etree.ElementTree
+        XML = self.etree.XML
+
+        tree = ElementTree( XML('<doc>This is a test.</doc>') )
+        self.assertRaises(IOError, tree.write,
+                          "definitely////\\-\\nonexisting\\-\\////FILE")
 
     # this could trigger a crash, apparently because the document
     # reference was prematurely garbage collected
@@ -634,6 +774,16 @@ class ETreeTestCaseBase(unittest.TestCase):
             a.text)
         self.assertXML('<a></a>', a)
         
+    def test_set_text_empty(self):
+        Element = self.etree.Element
+
+        a = Element('a')
+        self.assertEquals(None, a.text)
+
+        a.text = ''
+        self.assertEquals('', a.text)
+        self.assertXML('<a></a>', a)
+        
     def test_tail1(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -680,7 +830,7 @@ class ETreeTestCaseBase(unittest.TestCase):
             None,
             a.tail)
         self.assertXML('<a></a>', a)
-        
+
     def test_comment(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -688,9 +838,20 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         a = Element('a')
         a.append(Comment('foo'))
-        self.assertXML(
-            '<a><!-- foo --></a>',
-            a)
+        self.assertEquals(a[0].tag, Comment)
+        self.assertEquals(a[0].text, 'foo')
+
+    def test_comment_text(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        Comment = self.etree.Comment
+
+        a = Element('a')
+        a.append(Comment('foo'))
+        self.assertEquals(a[0].text, 'foo')
+
+        a[0].text = "TEST"
+        self.assertEquals(a[0].text, 'TEST')
         
     def test_comment_whitespace(self):
         Element = self.etree.Element
@@ -699,9 +860,7 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         a = Element('a')
         a.append(Comment(' foo  '))
-        self.assertXML(
-            '<a><!--  foo   --></a>',
-            a)
+        self.assertEquals(a[0].text, ' foo  ')
         
     def test_comment_nonsense(self):
         Comment = self.etree.Comment
@@ -715,6 +874,30 @@ class ETreeTestCaseBase(unittest.TestCase):
         for i in c:
             pass
 
+    def test_pi(self):
+        # lxml.etree separates target and text
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        ProcessingInstruction = self.etree.ProcessingInstruction
+
+        a = Element('a')
+        a.append(ProcessingInstruction('foo', 'some more text'))
+        self.assertEquals(a[0].tag, ProcessingInstruction)
+        self.assertXML("<a><?foo some more text?></a>",
+                       a)
+
+    def test_pi_nonsense(self):
+        ProcessingInstruction = self.etree.ProcessingInstruction
+        pi = ProcessingInstruction('foo')
+        self.assertEquals({}, pi.attrib)
+        self.assertEquals([], pi.keys())
+        self.assertEquals([], pi.items())
+        self.assertEquals(None, pi.get('hoi'))
+        self.assertEquals(0, len(pi))
+        # should not iterate
+        for i in pi:
+            pass
+
     def test_setitem(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -748,7 +931,17 @@ class ETreeTestCaseBase(unittest.TestCase):
             a)
         self.assertXML('<c></c>',
                        c)
-        
+
+    def test_setitem_replace(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        SubElement(a, 'b')
+        d = Element('d')
+        a[0] = d
+        self.assertXML('<a><d></d></a>', a)
+
     def test_setitem_indexerror(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -772,7 +965,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertXML(
             '<a><c></c>C2</a>',
             a)
-        
+
     def test_tag_write(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -790,6 +983,54 @@ class ETreeTestCaseBase(unittest.TestCase):
             '<c><b></b></c>',
             a)
 
+    def test_tag_reset_ns(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        tostring = self.etree.tostring
+
+        a = Element('{a}a')
+        b1 = SubElement(a, '{a}b')
+        b2 = SubElement(a, '{b}b')
+
+        self.assertEquals('{a}b',  b1.tag)
+
+        b1.tag = 'c'
+
+        # can't use C14N here!
+        self.assertEquals('c', b1.tag)
+        self.assertEquals('<c', tostring(b1)[:2])
+        self.assert_('<c' in tostring(a))
+
+    def test_tag_reset_root_ns(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        tostring = self.etree.tostring
+
+        a = Element('{a}a')
+        b1 = SubElement(a, '{a}b')
+        b2 = SubElement(a, '{b}b')
+
+        a.tag = 'c'
+
+        self.assertEquals(
+            'c',
+            a.tag)
+
+        # can't use C14N here!
+        self.assertEquals('c',  a.tag)
+        self.assertEquals('<c', tostring(a)[:2])
+
+    def test_tag_str_subclass(self):
+        Element = self.etree.Element
+
+        class strTest(str):
+            pass
+
+        a = Element("a")
+        a.tag = strTest("TAG")
+        self.assertXML('<TAG></TAG>',
+                       a)
+
     def test_delitem(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -1056,6 +1297,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertXML(
             '<a></a>',
             a)
+        self.assertEquals('b2', b.tail)
         
     def test_getchildren(self):
         Element = self.etree.Element
@@ -1105,6 +1347,26 @@ class ETreeTestCaseBase(unittest.TestCase):
             [d],
             list(d.getiterator()))
 
+    def test_getiterator_empty(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        e = SubElement(c, 'e')
+
+        self.assertEquals(
+            [],
+            list(a.getiterator('none')))
+        self.assertEquals(
+            [],
+            list(e.getiterator('none')))
+        self.assertEquals(
+            [e],
+            list(e.getiterator()))
+
     def test_getiterator_filter(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -1575,7 +1837,28 @@ class ETreeTestCaseBase(unittest.TestCase):
         # as namespace is not moved along with it
         del one
         self.assertEquals('{http://a.b.c}baz', baz.tag)
-        
+
+    def test_attribute_xmlns_move(self):
+        Element = self.etree.Element
+
+        root = Element('element')
+
+        subelement = Element('subelement',
+                             {"{http://www.w3.org/XML/1998/namespace}id": "foo"})
+        self.assertEquals(1, len(subelement.attrib))
+        self.assertEquals(
+            "foo",
+            subelement.get("{http://www.w3.org/XML/1998/namespace}id"))
+
+        root.append(subelement)
+        self.assertEquals(1, len(subelement.attrib))
+        self.assertEquals(
+            {"{http://www.w3.org/XML/1998/namespace}id" : "foo"}.items(),
+            subelement.attrib.items())
+        self.assertEquals(
+            "foo",
+            subelement.get("{http://www.w3.org/XML/1998/namespace}id"))
+
     def test_tostring(self):
         tostring = self.etree.tostring
         Element = self.etree.Element
@@ -1615,7 +1898,114 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         self.assert_(tostring(b) == '<b/>Foo' or
                      tostring(b) == '<b />Foo')
-        
+
+    def test_iterparse(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f)
+        self.assertEquals(None,
+                          iterator.root)
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('end', root[0]), ('end', root[1]), ('end', root)],
+            events)
+
+    def test_iterparse_file(self):
+        iterparse = self.etree.iterparse
+        iterator = iterparse(fileInTestDir("test.xml"))
+        self.assertEquals(None,
+                          iterator.root)
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('end', root[0]), ('end', root)],
+            events)
+
+    def test_iterparse_start(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f, events=('start',))
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('start', root), ('start', root[0]), ('start', root[1])],
+            events)
+
+    def test_iterparse_start_end(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f, events=('start','end'))
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('start', root), ('start', root[0]), ('end', root[0]),
+             ('start', root[1]), ('end', root[1]), ('end', root)],
+            events)
+
+    def test_iterparse_clear(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f)
+        for event, elem in iterator:
+            elem.clear()
+
+        root = iterator.root
+        self.assertEquals(0,
+                          len(root))
+
+    def test_iterparse_large(self):
+        iterparse = self.etree.iterparse
+        CHILD_COUNT = 12345
+        f = StringIO('<a>' + ('<b>test</b>'*CHILD_COUNT) + '</a>')
+
+        i = 0
+        for key in iterparse(f):
+            event, element = key
+            i += 1
+        self.assertEquals(i, CHILD_COUNT + 1)
+
+    def test_iterparse_attrib_ns(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a xmlns="ns1"><b><c xmlns="ns2"/></b></a>')
+
+        attr_name = '{testns}bla'
+        events = []
+        iterator = iterparse(f, events=('start','end','start-ns','end-ns'))
+        for event, elem in iterator:
+            events.append(event)
+            if event == 'start':
+                if elem.tag != '{ns1}a':
+                    elem.set(attr_name, 'value')
+
+        self.assertEquals(
+            ['start-ns', 'start', 'start', 'start-ns', 'start',
+             'end', 'end-ns', 'end', 'end', 'end-ns'],
+            events)
+
+        root = iterator.root
+        self.assertEquals(
+            None,
+            root.get(attr_name))
+        self.assertEquals(
+            'value',
+            root[0].get(attr_name))
+
+    def test_iterparse_getiterator(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b><d/></b><c/></a>')
+
+        counts = []
+        for event, elem in iterparse(f):
+            counts.append(len(list(elem.getiterator())))
+        self.assertEquals(
+            [1,2,1,4],
+            counts)
+
     def test_parse_file(self):
         parse = self.etree.parse
         # from file
@@ -1627,7 +2017,7 @@ class ETreeTestCaseBase(unittest.TestCase):
     def test_parse_file_nonexistent(self):
         parse = self.etree.parse
         self.assertRaises(IOError, parse, fileInTestDir('notthere.xml'))  
-        
+
     def test_parse_file_object(self):
         parse = self.etree.parse
         # from file object
@@ -1648,28 +2038,24 @@ class ETreeTestCaseBase(unittest.TestCase):
             '<a><b></b></a>',
             tree.getroot()
            )
-        
+
+    def test_parse_with_encoding(self):
+        # this can fail in libxml2 <= 2.6.22
+        parse = self.etree.parse
+        tree = parse(StringIO('<?xml version="1.0" encoding="ascii"?><html/>'))
+        self.assertXML('<html></html>',
+                       tree.getroot())
+
     def test_encoding(self):
-        ElementTree = self.etree.ElementTree
         Element = self.etree.Element
 
         a = Element('a')
         a.text = u'Søk på nettet'
         self.assertXML(
             u'<a>Søk på nettet</a>'.encode('UTF-8'),
-            a)
+            a, 'utf-8')
 
-    def test_encoding2(self):
-        ElementTree = self.etree.ElementTree
-        Element = self.etree.Element
-
-        a = Element('a')
-        a.text = u'Søk på nettet'
-        self.assertXML(
-            u'<a>Søk på nettet</a>'.encode('UTF-8'),
-            a, 'UTF-8')
-        
-    def test_encoding3(self):
+    def test_encoding_exact(self):
         ElementTree = self.etree.ElementTree
         Element = self.etree.Element
 
@@ -1678,25 +2064,54 @@ class ETreeTestCaseBase(unittest.TestCase):
         
         f = StringIO()
         tree = ElementTree(element=a)
-        tree.write(f, 'UTF-8')
-        data = f.getvalue()
+        tree.write(f, 'utf-8')
+        self.assertEquals(u'<a>Søk på nettet</a>'.encode('UTF-8'),
+                          f.getvalue())
 
-        # XXX prologue generation seems to be inconsistent between libraries..
-        xml = u'<a>Søk på nettet</a>'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
-        self.assert_(data in [xml, prologue + xml])
+    def test_parse_file_encoding(self):
+        parse = self.etree.parse
+        # from file
+        tree = parse(fileInTestDir('test-string.xml'))
+        self.assertXML(
+            u'<a>Søk på nettet</a>'.encode('UTF-8'),
+            tree.getroot(), 'UTF-8')
 
-##     # ignore wrong (left-over?) encoding declaration in unicode strings
-##     def _test_wrong_unicode_encoding(self):
-##         XML = self.etree.XML
+    def test_parse_file_object_encoding(self):
+        parse = self.etree.parse
+        # from file object
+        f = open(fileInTestDir('test-string.xml'), 'r')
+        tree = parse(f)
+        f.close()
+        self.assertXML(
+            u'<a>Søk på nettet</a>'.encode('UTF-8'),
+            tree.getroot(), 'UTF-8')
 
-##         test_utf = u'<?xml version=\'1.0\' encoding=\'iso-8859-1\'?><a>Søk på nettet</a>'
-##         parsed = XML(test_utf)
-##         self.assertXML(
-##             u'<a>Søk på nettet</a>'.encode('UTF-8'),
-##             parsed, 'UTF-8')
-        
-    def test_encoding_default_encoding(self):
+    def test_encoding_8bit_latin1(self):
+        ElementTree = self.etree.ElementTree
+        Element = self.etree.Element
+
+        a = Element('a')
+        a.text = u'Søk på nettet'
+
+        f = StringIO()
+        tree = ElementTree(element=a)
+        tree.write(f, 'iso-8859-1')
+        result = f.getvalue()
+        declaration = "<?xml version=\'1.0\' encoding=\'iso-8859-1\'?>"
+        self.assertEncodingDeclaration(result,'iso-8859-1')
+        result = result.split('?>', 1)[-1]
+        if result[0] == '\n':
+            result = result[1:]
+        self.assertEquals(u'<a>Søk på nettet</a>'.encode('iso-8859-1'),
+                          result)
+
+    # raise error on wrong (left-over?) encoding declaration in unicode strings
+    def _test_wrong_unicode_encoding(self):
+        XML = self.etree.XML
+        test_utf = u'<?xml version=\'1.0\' encoding=\'iso-8859-1\'?><a>Søk på nettet</a>'
+        self.assertRaises(SyntaxError, XML, test_utf)
+
+    def test_encoding_write_default_encoding(self):
         ElementTree = self.etree.ElementTree
         Element = self.etree.Element
 
@@ -1708,20 +2123,25 @@ class ETreeTestCaseBase(unittest.TestCase):
         tree.write(f)
         data = f.getvalue()
         self.assertEquals(
-            '<a>S&#248;k p&#229; nettet</a>',
+            u'<a>Søk på nettet</a>'.encode('ASCII', 'xmlcharrefreplace'),
             data)
 
     def test_encoding_tostring(self):
         Element = self.etree.Element
         tostring = self.etree.tostring
 
-        # XXX prologue generation seems to be inconsistent between libraries..
-        xml = u'<a>Søk på nettet</a>'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
+        a = Element('a')
+        a.text = u'Søk på nettet'
+        self.assertEquals(u'<a>Søk på nettet</a>'.encode('UTF-8'),
+                         tostring(a, 'utf-8'))
+
+    def test_encoding_tostring_unknown(self):
+        Element = self.etree.Element
+        tostring = self.etree.tostring
         
         a = Element('a')
         a.text = u'Søk på nettet'
-        self.assert_(tostring(a, 'UTF-8') in [xml, prologue + xml])
+        self.assertRaises(LookupError, tostring, a, 'Invalid Encoding')
 
     def test_encoding_tostring_sub(self):
         Element = self.etree.Element
@@ -1731,11 +2151,8 @@ class ETreeTestCaseBase(unittest.TestCase):
         a = Element('a')
         b = SubElement(a, 'b')
         b.text = u'Søk på nettet'
-
-        # XXX prologue generation seems to be inconsistent between libraries..
-        xml = u'<b>Søk på nettet</b>'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
-        self.assert_(tostring(b, 'UTF-8') in [xml, prologue + xml])
+        self.assertEquals(u'<b>Søk på nettet</b>'.encode('UTF-8'),
+                         tostring(b, 'utf-8'))
 
     def test_encoding_tostring_sub_tail(self):
         Element = self.etree.Element
@@ -1746,9 +2163,8 @@ class ETreeTestCaseBase(unittest.TestCase):
         b = SubElement(a, 'b')
         b.text = u'Søk på nettet'
         b.tail = u'Søk'
-        xml = u'<b>Søk på nettet</b>Søk'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
-        self.assert_(tostring(b, 'UTF-8') in [xml, prologue + xml])
+        self.assertEquals(u'<b>Søk på nettet</b>Søk'.encode('UTF-8'),
+                         tostring(b, 'utf-8'))
         
     def test_encoding_tostring_default_encoding(self):
         Element = self.etree.Element
@@ -1772,10 +2188,40 @@ class ETreeTestCaseBase(unittest.TestCase):
         b = SubElement(a, 'b')
         b.text = u'Søk på nettet'
 
-        # the same, just hex versus decimal
         expected = '<b>S&#248;k p&#229; nettet</b>'
-        expected2 = '<b>S&#xF8;k p&#xE5; nettet</b>'
-        self.assert_(tostring(b) in [expected, expected2])
+        self.assertEquals(
+            expected,
+            tostring(b))
+
+    def test_encoding_8bit_xml(self):
+        utext = u'Søk på nettet'
+        uxml = u'<p>%s</p>' % utext
+        prologue = '<?xml version="1.0" encoding="iso-8859-1" ?>'
+        isoxml = prologue + uxml.encode('iso-8859-1')
+        tree = self.etree.XML(isoxml)
+        self.assertEquals(utext, tree.text)
+
+    def test_encoding_8bit_parse_stringio(self):
+        utext = u'Søk på nettet'
+        uxml = u'<p>%s</p>' % utext
+        prologue = '<?xml version="1.0" encoding="iso-8859-1" ?>'
+        isoxml = prologue + uxml.encode('iso-8859-1')
+        el = self.etree.parse(StringIO(isoxml)).getroot()
+        self.assertEquals(utext, el.text)
+
+    def test_deepcopy_elementtree(self):
+        Element = self.etree.Element
+        ElementTree = self.etree.ElementTree
+
+        a = Element('a')
+        a.text = "Foo"
+        atree = ElementTree(a)
+
+        btree = copy.deepcopy(atree)
+        self.assertEqual("Foo", atree.getroot().text)
+        self.assertEqual("Foo", btree.getroot().text)
+        self.assertFalse(btree is atree)
+        self.assertFalse(btree.getroot() is atree.getroot())
 
     def test_deepcopy(self):
         Element = self.etree.Element
@@ -1790,6 +2236,75 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('Bar', b.text)
         self.assertEquals('Foo', a.text)
 
+        del a
+        self.assertEquals('Bar', b.text)
+
+    def test_deepcopy_tail(self):
+        Element = self.etree.Element
+        
+        a = Element('a')
+        a.tail = 'Foo'
+
+        b = copy.deepcopy(a)
+        self.assertEquals('Foo', b.tail)
+        
+        b.tail = 'Bar'
+        self.assertEquals('Bar', b.tail)
+        self.assertEquals('Foo', a.tail)
+
+        del a
+        self.assertEquals('Bar', b.tail)
+
+    def test_deepcopy_subelement(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        root = Element('root')
+        a = SubElement(root, 'a')
+        a.text = 'FooText'
+        a.tail = 'FooTail'
+
+        b = copy.deepcopy(a)
+        self.assertEquals('FooText', b.text)
+        self.assertEquals('FooTail', b.tail)
+        
+        b.text = 'BarText'
+        b.tail = 'BarTail'
+        self.assertEquals('BarTail', b.tail)
+        self.assertEquals('FooTail', a.tail)
+        self.assertEquals('BarText', b.text)
+        self.assertEquals('FooText', a.text)
+
+        del a
+        self.assertEquals('BarTail', b.tail)
+        self.assertEquals('BarText', b.text)
+
+    def test_deepcopy_namespaces(self):
+        root = self.etree.XML('''<doc xmlns="dns" xmlns:t="tns">
+        <parent><node t:foo="bar" /></parent>
+        </doc>''')
+        self.assertEquals(
+            root[0][0].get('{tns}foo'),
+            copy.deepcopy(root[0])[0].get('{tns}foo') )
+        self.assertEquals(
+            root[0][0].get('{tns}foo'),
+            copy.deepcopy(root[0][0]).get('{tns}foo') )
+        
+    def test_deepcopy_append(self):
+        # previously caused a crash
+        Element = self.etree.Element
+        tostring = self.etree.tostring
+        
+        a = Element('a')
+        b = copy.deepcopy(a)
+        a.append( Element('C') )
+        b.append( Element('X') )
+
+        self.assertEquals('<a><C/></a>',
+                          tostring(a).replace(' ', ''))
+        self.assertEquals('<a><X/></a>',
+                          tostring(b).replace(' ', ''))
+
     def test_shallowcopy(self):
         Element = self.etree.Element
         
@@ -1803,7 +2318,20 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('Bar', b.text)
         self.assertEquals('Foo', a.text)
         # XXX ElementTree will share nodes, but lxml.etree won't..
+
+    def test_shallowcopy_elementtree(self):
+        Element = self.etree.Element
+        ElementTree = self.etree.ElementTree
         
+        a = Element('a')
+        a.text = 'Foo'
+        atree = ElementTree(a)
+
+        btree = copy.copy(atree)
+        self.assertFalse(btree is atree)
+        self.assert_(btree.getroot() is atree.getroot())
+        self.assertEquals('Foo', atree.getroot().text)
+
     def test_element_boolean(self):
         etree = self.etree
         e = etree.Element('foo')
@@ -1846,6 +2374,15 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals(a1.tag, "{myns}a")
         self.assertEquals(a2.tag, "{myns}a")
 
+    def test_qname_cmp(self):
+        etree = self.etree
+        qname1 = etree.QName('myns', 'a')
+        qname2 = etree.QName('myns', 'a')
+        self.assertEquals(qname1, "{myns}a")
+        self.assertEquals("{myns}a", qname2)
+        self.assertEquals(qname1, qname1)
+        self.assertEquals(qname1, qname2)
+
     def _writeElement(self, element, encoding='us-ascii'):
         """Write out element for comparison.
         """
@@ -1861,15 +2398,17 @@ class ETreeTestCaseBase(unittest.TestCase):
         """
         ElementTree = self.etree.ElementTree
         handle, filename = tempfile.mkstemp()
-        f = open(filename, 'wb')
-        tree = ElementTree(element=element)
-        tree.write(f, encoding)
-        f.close()
-        f = open(filename, 'rb')
-        data = unicode(f.read(), encoding)
-        f.close()
-        os.close(handle)
-        os.remove(filename)
+        try:
+            f = open(filename, 'wb')
+            tree = ElementTree(element=element)
+            tree.write(f, encoding)
+            f.close()
+            f = open(filename, 'rb')
+            data = unicode(f.read(), encoding)
+            f.close()
+        finally:
+            os.close(handle)
+            os.remove(filename)
         return canonicalize(data)
 
     def assertXML(self, expected, element, encoding='us-ascii'):
@@ -1879,6 +2418,13 @@ class ETreeTestCaseBase(unittest.TestCase):
         """
         self.assertEquals(expected, self._writeElement(element, encoding))
         self.assertEquals(expected, self._writeElementFile(element, encoding))
+
+    def assertEncodingDeclaration(self, result, encoding):
+        "Checks if the result XML byte string specifies the encoding."
+        has_encoding = re.compile(r"<\?xml[^>]+ encoding=[\"']([^\"']+)[\"']").match
+        self.assert_(has_encoding(result))
+        result_encoding = has_encoding(result).group(1)
+        self.assertEquals(result_encoding.upper(), encoding.upper())
         
     def _rootstring(self, tree):
         return self.etree.tostring(tree.getroot()).replace(' ', '').replace('\n', '')
