@@ -8,10 +8,29 @@ test_elementtree
 """
 
 
-import unittest, doctest
+import unittest, copy, sys
 
 from common_imports import etree, StringIO, HelperTestCase, fileInTestDir
-from common_imports import SillyFileLike, canonicalize
+from common_imports import SillyFileLike, canonicalize, doctest
+
+print
+print "TESTED VERSION:"
+print "    Python:           ", sys.version_info
+print "    lxml.etree:       ", etree.LXML_VERSION
+print "    libxml used:      ", etree.LIBXML_VERSION
+print "    libxml compiled:  ", etree.LIBXML_COMPILED_VERSION
+print "    libxslt used:     ", etree.LIBXSLT_VERSION
+print "    libxslt compiled: ", etree.LIBXSLT_COMPILED_VERSION
+print
+
+try:
+    sorted(())
+except NameError:
+    # Python 2.3
+    def sorted(seq):
+        seq = list(seq)
+        seq.sort()
+        return seq
 
 class ETreeOnlyTestCase(HelperTestCase):
     """Tests only for etree, not ElementTree"""
@@ -51,20 +70,6 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals("TEST", root.get("attr"))
         self.assertRaises(TypeError, root.set, "newattr", 5)
 
-    def test_parse_file_dtd(self):
-        parse = self.etree.parse
-        parser = self.etree.XMLParser(attribute_defaults=True)
-
-        tree = parse(fileInTestDir('test.xml'), parser)
-        root = tree.getroot()
-
-        self.assertEquals(
-            "valueA",
-            root.get("default"))
-        self.assertEquals(
-            "valueB",
-            root[0].get("default"))
-
     def test_pi(self):
         # lxml.etree separates target and text
         Element = self.etree.Element
@@ -82,6 +87,29 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(root[0].target, "mypi")
         self.assertEquals(root[0].text, "my test ")
 
+    def test_deepcopy_pi(self):
+        # previously caused a crash
+        ProcessingInstruction = self.etree.ProcessingInstruction
+        
+        a = ProcessingInstruction("PI", "ONE")
+        b = copy.deepcopy(a)
+        b.text = "ANOTHER"
+
+        self.assertEquals('ONE',     a.text)
+        self.assertEquals('ANOTHER', b.text)
+
+    def test_deepcopy_comment(self):
+        # previously caused a crash
+        # not supported by ET!
+        Comment = self.etree.Comment
+        
+        a = Comment("ONE")
+        b = copy.deepcopy(a)
+        b.text = "ANOTHER"
+
+        self.assertEquals('ONE',     a.text)
+        self.assertEquals('ANOTHER', b.text)
+
     def test_attribute_set(self):
         # ElementTree accepts arbitrary attribute values
         # lxml.etree allows only strings
@@ -136,16 +164,6 @@ class ETreeOnlyTestCase(HelperTestCase):
         # ET raises ExpatError, lxml raises XMLSyntaxError
         self.assertRaises(self.etree.XMLSyntaxError, list, iterparse(f))
 
-    def test_iterparse_file_dtd(self):
-        iterparse = self.etree.iterparse
-        iterator = iterparse(fileInTestDir("test.xml"), events=("start",),
-                             attribute_defaults=True)
-        attributes = [ element.get("default")
-                       for event, element in iterator ]
-        self.assertEquals(
-            ["valueA", "valueB"],
-            attributes)
-
     def test_iterparse_strip(self):
         iterparse = self.etree.iterparse
         f = StringIO("""
@@ -851,6 +869,81 @@ class ETreeOnlyTestCase(HelperTestCase):
             '<z xmlns="http://ns.infrae.com/foo" xmlns:hoi="http://ns.infrae.com/hoi"><hoi:x></hoi:x></z>',
             self._writeElement(e))
 
+    def test_namespaces_default_copy_element(self):
+        etree = self.etree
+
+        r = {None: 'http://ns.infrae.com/foo'}
+        e1 = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=r)
+        e2 = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=r)
+
+        e1.append(e2)
+
+        self.assertEquals(
+            None,
+            e1.prefix)
+        self.assertEquals(
+            None,
+            e1[0].prefix)
+        self.assertEquals(
+            '{http://ns.infrae.com/foo}bar',
+            e1.tag)
+        self.assertEquals(
+            '{http://ns.infrae.com/foo}bar',
+            e1[0].tag)
+
+    def test_namespaces_copy_element(self):
+        etree = self.etree
+
+        r = {None: 'http://ns.infrae.com/BAR'}
+        e1 = etree.Element('{http://ns.infrae.com/BAR}bar', nsmap=r)
+        e2 = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=r)
+
+        e1.append(e2)
+
+        self.assertEquals(
+            None,
+            e1.prefix)
+        self.assertNotEquals(
+            None,
+            e2.prefix)
+        self.assertEquals(
+            '{http://ns.infrae.com/BAR}bar',
+            e1.tag)
+        self.assertEquals(
+            '{http://ns.infrae.com/foo}bar',
+            e2.tag)
+
+    def test_namespaces_reuse_after_move(self):
+        ns_href = "http://a.b.c"
+        one = self.etree.parse(
+            StringIO('<foo><bar xmlns:ns="%s"><ns:baz/></bar></foo>' % ns_href))
+        baz = one.getroot()[0][0]
+
+        two = self.etree.parse(
+            StringIO('<root xmlns:ns="%s"/>' % ns_href))
+        two.getroot().append(baz)
+        del one # make sure the source document is deallocated
+
+        self.assertEquals('{%s}baz' % ns_href, baz.tag)
+        self.assertEquals(
+            '<root xmlns:ns="%s"><ns:baz/></root>' % ns_href,
+            self.etree.tostring(two))
+
+    def _test_namespaces_after_serialize(self):
+        # FIXME: this currently fails - fix serializer.pxi!
+        parse = self.etree.parse
+        tostring = self.etree.tostring
+
+        ns_href = "http://a.b.c"
+        one = parse(
+            StringIO('<foo><bar xmlns:ns="%s"><ns:baz/></bar></foo>' % ns_href))
+        baz = one.getroot()[0][0]
+
+        print tostring(baz)
+        parsed = parse(StringIO( tostring(baz) )).getroot()
+
+        self.assertEquals('{%s}baz' % ns_href, parsed.tag)
+
     def test_element_nsmap(self):
         etree = self.etree
 
@@ -984,13 +1077,48 @@ class ETreeOnlyTestCase(HelperTestCase):
             etree.SubElement(e, 'a%s' % i)
 
         new_element = etree.Element("test")
+        new_element.text = "TESTTEXT"
+        new_element.tail = "TESTTAIL"
         child1 = e[1]
         e.replace(e[0], new_element)
         self.assertEquals(
             new_element, e[0])
+        self.assertEquals(
+            "TESTTEXT",
+            e[0].text)
+        self.assertEquals(
+            "TESTTAIL",
+            e[0].tail)
         self.assertEquals(
             child1, e[1])
 
+    def test_extend(self):
+        etree = self.etree
+        root = etree.Element('foo')
+        for i in range(3):
+            element = etree.SubElement(root, 'a%s' % i)
+            element.text = "text%d" % i
+            element.tail = "tail%d" % i
+
+        elements = []
+        for i in range(3):
+            new_element = etree.Element("test%s" % i)
+            new_element.text = "TEXT%s" % i
+            new_element.tail = "TAIL%s" % i
+            elements.append(new_element)
+
+        root.extend(elements)
+
+        self.assertEquals(
+            ["a0", "a1", "a2", "test0", "test1", "test2"],
+            [ el.tag for el in root ])
+        self.assertEquals(
+            ["text0", "text1", "text2", "TEXT0", "TEXT1", "TEXT2"],
+            [ el.text for el in root ])
+        self.assertEquals(
+            ["tail0", "tail1", "tail2", "TAIL0", "TAIL1", "TAIL2"],
+            [ el.tail for el in root ])
+
     def test_docinfo_public(self):
         etree = self.etree
         xml_header = '<?xml version="1.0" encoding="ascii"?>'
@@ -1151,16 +1279,42 @@ class ETreeOnlyTestCase(HelperTestCase):
         return canonicalize(data)
 
 
-class ETreeXIncludeTestCase(HelperTestCase):
+class XIncludeTestCase(HelperTestCase):
+    def test_xinclude_text(self):
+        filename = fileInTestDir('test_broken.xml')
+        root = etree.XML('''\
+        <doc xmlns:xi="http://www.w3.org/2001/XInclude">
+          <xi:include href="%s" parse="text"/>
+        </doc>
+        ''' % filename)
+        old_text = root.text
+        content = open(filename).read()
+        old_tail = root[0].tail
+
+        self.include( etree.ElementTree(root) )
+        self.assertEquals(old_text + content + old_tail,
+                          root.text)
+
+class ETreeXIncludeTestCase(XIncludeTestCase):
+    def include(self, tree):
+        tree.xinclude()
+
     def test_xinclude(self):
         tree = etree.parse(fileInTestDir('test_xinclude.xml'))
         # process xincludes
-        tree.xinclude()
+        self.include( tree )
         # check whether we find it replaced with included data
         self.assertEquals(
             'a',
             tree.getroot()[1].tag)
-        
+
+
+class ElementIncludeTestCase(XIncludeTestCase):
+    from lxml import ElementInclude
+    def include(self, tree):
+        self.ElementInclude.include(tree.getroot())
+
+
 class ETreeC14NTestCase(HelperTestCase):
     def test_c14n(self):
         tree = self.parse('<a><b/></a>')
@@ -1174,9 +1328,12 @@ def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ETreeOnlyTestCase)])
     suite.addTests([unittest.makeSuite(ETreeXIncludeTestCase)])
+    suite.addTests([unittest.makeSuite(ElementIncludeTestCase)])
     suite.addTests([unittest.makeSuite(ETreeC14NTestCase)])
     suite.addTests(
         [doctest.DocFileSuite('../../../doc/api.txt')])
+    suite.addTests(
+        [doctest.DocFileSuite('../../../doc/parsing.txt')])
     suite.addTests(
         [doctest.DocFileSuite('../../../doc/resolvers.txt')])
     return suite
