@@ -74,7 +74,7 @@ class ETreeXPathTestCase(HelperTestCase):
 
     def test_xpath_list_comment(self):
         tree = self.parse('<a><!-- Foo --></a>')
-        self.assertEquals(['<Comment[ Foo ]>'],
+        self.assertEquals(['<!-- Foo -->'],
                           map(repr, tree.xpath('/a/node()')))
 
     def test_rel_xpath_boolean(self):
@@ -503,6 +503,8 @@ def test_suite():
     suite.addTests([unittest.makeSuite(ETreeXPathClassTestCase)])
     suite.addTests([unittest.makeSuite(ETreeETXPathClassTestCase)])
     suite.addTests([doctest.DocTestSuite()])
+    suite.addTests(
+        [doctest.DocFileSuite('../../../doc/xpathxslt.txt')])
     return suite
 
 if __name__ == '__main__':
